var jwt = require('jsonwebtoken');
var bcrypt = require('bcryptjs');
var config = require('../helpers/config.js');
var dbPool = require('../helpers/db-pool.js').pool;


exports.getAll = function(cb) {
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT * FROM USER", cb);
        connection.release();
    })

};

exports.getOne = function(id, cb) {
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT * FROM USER WHERE UserID = ?", id , cb);
        connection.release();
    })
};

exports.getType = function(role, cb) {
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT * FROM USER WHERE Role = ?", role , cb);
        connection.release();
    })
};

exports.getByPatientID = function(patientID, cb) {
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT USER.* FROM USER INNER JOIN ACCESS ON USER.UserID = ACCESS.UserID WHERE ACCESS.PatientID = ? AND ACCESS.Permission = 1", patientID , cb);
        connection.release();
    })
};

exports.getPatientForFamily = function(userId, cb) {
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT VISIT.VisitID, VISIT.UserID, VISIT.PatientID, VISIT.DateTime, USER.FirstName, USER.LastName FROM VISIT INNER JOIN ACCESS ON VISIT.PatientID = ACCESS.PatientID INNER JOIN USER ON VISIT.UserID = USER.UserID WHERE ACCESS.UserID = ? AND ACCESS.Permission = 1", userId, cb);
        connection.release();
    })
};

exports.getGroupMembers = function(userId, cb) {
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT USER.UserID, USER.FirstName, USER.LastName\n" +
            "FROM USER\n" +
            "INNER JOIN ACCESS ON USER.UserID = ACCESS.UserID AND ACCESS.Permission = 1\n" +
            "WHERE ACCESS.PatientID IN (SELECT ACCESS.PatientID FROM ACCESS WHERE ACCESS.UserID = ? AND ACCESS.Permission = 1)\n" +
            "AND USER.UserID != ?", [userId, userId], cb);
        connection.release();
    })
};

exports.updateUserPermission = function(userId, requestBody, cb) {
    var permission = JSON.parse(requestBody.data);
    dbPool.getConnection(function(err, connection) {
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("UPDATE ACCESS SET Permission = ? WHERE UserID = ?",
            [permission.permission, permission.userId], function (err, result) {
                if (err) {
                    connection.release();
                    cb(err, null);
                    throw err;
                }
                connection.release();
            })
    })
};


exports.insertNewUser = function(requestBody, cb) {
    var userDetails = JSON.parse(requestBody.data);
    var hashedPassword = bcrypt.hashSync(userDetails.password, 8);
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        var role = userDetails.role;
        connection.query("INSERT INTO USER(FirstName, LastName, Email, Password, Role) VALUES (?, ?, ?, ?, ? )", [userDetails.firstName, userDetails.lastName, userDetails.email, hashedPassword, role], function(err, result) {
            if (err) {
                if (err.code === 'ER_DUP_ENTRY') {
                    cb(null, {auth: false, token: null, role: null});
                } else {
                    throw err;
                }
            } else {
                var userId = result.insertId; //result object gives the ID of the newly inserted visit
                // create a token
                var token = jwt.sign({id: userId, role: userDetails.role}, config.secret, {
                    expiresIn: 86400 // expires in 24 hours
                });
                cb(null, {auth: true, token: token, role: role});
            }
        });
        connection.release();
    });
};

exports.login = function(requestBody, cb) {
    var loginDetails = JSON.parse(requestBody.data);
    dbPool.getConnection(function(err, connection) {
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("SELECT * FROM USER WHERE Email = ?", loginDetails.email, function (err, result) {
            if (result.length === 1) {
                var hashedPassword = result[0].Password;
                var userId = result[0].UserID;
                var role = result[0].Role;
                var passwordIsValid = bcrypt.compareSync(loginDetails.password, hashedPassword);
                if (passwordIsValid) {
                    // create a token
                    var token = jwt.sign({id: userId, role: role}, config.secret, {
                        expiresIn: 86400 // expires in 24 hours
                    });
                    cb(null, {auth: true, token: token, role: role});
                } else {
                    cb(null, {auth: false, token: null, role: null});
                }
            } else {
                cb(null, {auth: false, token: null, role: null});
            }
        });
        connection.release();
    });
};

exports.assignMemberToPatient = function(userId, requestBody, cb) {
    var patient = JSON.parse(requestBody.data);
    var uuid = patient.uuid;
    var patientLastName = patient.lastName;
    dbPool.getConnection(function(err, connection){
        if (err) {
            connection.release();
            cb(err, null);
            throw err;
        }
        connection.query("INSERT INTO ACCESS(PatientID, UserID, Permission) SELECT PatientID, ?, 1 FROM PATIENT WHERE UuID = ? AND LastName = ?", [userId, uuid, patientLastName], function(err, result) {
                if (err) {
                    connection.release();
                    cb(err, null);
                    throw err;
                }
                if (result.affectedRows === 1) {
                    cb(null, result);
                } else {
                    cb(404, null);
                }
            });
        connection.release();
    })

};



